/* An example of FUNCDEF'ing various Windows OS functions
 * to load and draw an icon in a REXX Dialog window.
 *
 * NOTE: Assumes REXX Dialog is autoloaded.
 */

/* Note: Since we're using Reginald exclusively, we'll also use
 * ADDRESS NULL so that we can eliminate the need for the CALL
 * keyword.
 */
ADDRESS NULL
NUMERIC DIGITS 10








/* ============== FUNCDEF some needed OS functions ============ */

/* Make things easy for us with WINFUNC option */
OPTIONS "WINFUNC"

/* Trap ERROR, and ask REXX Dialog for ERROR raising */
SIGNAL ON ERROR
SIGNAL ON FAILURE
RXERR('ERROR')

/* Note: Because we trap ERROR, if any of the FUNCDEF's below fail,
 * then we'll jump to our label ERROR
 */

/* Register LoadImage */
FUNCDEF('LoadImage', '32u, 32u, str, 32u, 32u, 32u, 32u', 'user32')

/* Register DestroyIcon */
FUNCDEF("DestroyIcon", "32, void", 'user32')

/* Register DrawIconEx */
FUNCDEF("DrawIconEx","32,void,32,32,void,32,32,32u,void,32u","user32")

/* Register BeginPaint and EndPaint */
FUNCDEF("BeginPaint","32u,void,void stor","user32")
FUNCDEF("EndPaint","32,void,void","user32")

/* Register InvalidateRect */
RECT = "32,32,32,32"
FUNCDEF("InvalidateRect","32,void,struct RECT,32","user32")






/* ================= Load an icon (image) =================== */
/* We load the image from the file MyIcon.ico in the current
 * directory. We need to load it before we create our window
 * because we need to draw it in our window.
 */
hIcon = LoadImage(0, "MyIcon.ico", 1 /* IMAGE_ICON */, 16, 16, 16 /* LR_LOADFROMFILE */)
IF hIcon == 0 THEN DO
    RXSAY("Error loading MyIcon.ico")
    RETURN
END





/* ================ Create a REXX Dialog window ================ */
/* First Group will be a text group with no groupbox. */
RXTYPE.1 = 'TEXT'
RXFLAGS.1 = 'NOBORDER'
RXLABEL.1 = "Here's the icon||"
RXPOS.1 = '1 10 10'
RX = ''
/* Note: Subclass the window to call our subroutine named "MyWindowProc" */
RXCREATE('RX', 1, 'Icon display example', 'MIN',,,"MyWindowProc")







/* ===================== Do user interaction ==================== */

/* Force a window repaint now that we have added an icon to it */
InvalidateRect(WindowHandle)

DO FOREVER

   /* NOTE: While we're inside this call to RXMSG(), and some event
    * happens with the above window, then our MyWindowProc
    * function gets called.
    */
   RXMSG()

   /* Did user click on the CLOSE BOX? */
   IF RXID == '' THEN SIGNAL HALT

END









/* ============== Window Procedure written in REXX ==============
 * There are at least 2 args passed to this procedure. See
 * subclass.rex
 */

MyWindowProc:

   /* Create a label by combining the Window ID followed by
    * an underscore, and then the message. For example, if
    * our Window ID is 'RX', and the message we receive is
    * 'MOUSEMOVE', then the label we'll create is RX_MOUSEMOVE.
    */
   label = ARG(1) || '_' || ARG(2)   

   /* Now check to see if we have such a label in our script.
    * If so, then we have a subroutine that is intended to
    * handle that specific message for this particular window.
    */
   IF EXISTS(label, 'L') THEN

      /* It exist so just just to it now. It has access to
       * the same args passed to the window procedure.
       */
      SIGNAL VALUE label

   /* Otherwise, we don't have a subroutine to handle this
    * specific message for this particular window, so we
    * must return no value to tell REXX Dialog to do the
    * default handling.
    */
   RETURN




/* ================== RX_PAINT ==================
 * This handles the PAINT message for my 'RX' window.
 *
 * The OS gives us this message when it wants us to paint
 * some graphics in the window.
 *
 * The first two args are the same as passed to MyWindowProc.
 *
 * ARG(3) is our device context handle.
 * ARG(4) is the handle to the entry control (window).
 *
 *
 * NOTE: Since we SIGNAL here (from MyWindowProc), this is
 * not a real subroutine. Don't use PROCEDURE here. Instead,
 * use PROCEDURE on MyWindowProc if that's what you desire.
 */

RX_CTLCOLORSTATIC:

  /* Before we can begin, we need to get an HDC */
  RXQUERY(ARG(1), 'HANDLE', window)
  hdc = BeginPaint(window, paintStruct)

  /* Draw the icon at an X position of 110 and y position of 10 */
  DrawIconEx(hdc, 110, 10, hIcon, , , , , 8 + 1 + 2)

  /* Finished with painting */
  EndPaint(window, hdc)

  /* If we handle PAINT we return an empty string */
  RETURN ""







/* ======================= Error Handling ======================= */
SYNTAX:
ERROR:
FAILURE:
   /* NOTE: CONDITION('D') fetches error message. CONDITION('E') fetches the
    * error number. SIGL is the line number where the error occurred.
    */
   CONDITION('M')

HALT:
   RXMSG(,'END')

EndMe:
   /* Destroy the icon */
   IF EXISTS(hIcon) THEN DestroyIcon(hIcon)

   RETURN