/* An example of FUNCDEF'ing various Windows OS functions
 * to read email. This uses simple MAPI, which utilizes
 * whatever email client has been setup on the computer.
 */

OPTIONS "WINFUNC NOSOURCE C_CALL"
NUMERIC DIGITS 10






/* ============== FUNCDEF some needed OS functions ============== */
DO
	/* Register a MapiRecipDesc struct. Note: Comment out this line if you FUNCDEF any
	 * MAPI function that specifies a MapiMessage or MapiRecipDesc as an arg passed
	 * to it.
	 */
	FUNCDEF("MapiRecipDesc", "32u, 32u, str *, str *, 32u, void")

	/* Register the Windows OS function MAPIReadMail() */
	FUNCDEF("MAPIReadMail", "32u, void, void, str, 32u, 32u, void stor", "mapi32", "MAPIReadMail")

	/* Register the Windows OS function MAPIFindNext() */
	FUNCDEF("MAPIFindNext", "32u, void, void, str, str, 32u, 32u, str[513] stor", "mapi32")

	/* Register the Windows OS function MAPILogon() */
	FUNCDEF("MAPILogon", "32u, void, str, str, 32u, 32u, void stor", "mapi32")

	/* Register the Windows OS function MAPILogoff() */
	FUNCDEF("MAPILogoff", "32u, void, void, 32u, 32u", "mapi32")

	/* Register the Windows OS function MAPIFreeBuffer() */
	FUNCDEF("MAPIFreeBuffer", "32u, void", "mapi32")

	CATCH FAILURE
		CONDITION("M")
		RETURN
END





/* ========================= Read emails ========================= */

/* Log on (to the internet). We need to do this before receiving email.
 * This will bring up a connection requester for the email
 * client if the user is not yet connected to the internet.
 *
 * The first arg is a handle to some window you've opened. Omit this
 * if you haven't opened a window. If you have no window, then any
 * dialog presented (such as a connection dialog) is modal -- that is,
 * the user will have to operate it before he can access another program
 * on his computer. So, it's best to open a REXX Dialog window and use
 * RXQUERY "HANDLE" to get a handle.
 *
 * The second arg is a profile name string, limited to 256 characters or
 * less. This is the profile to use when logging on. If you omit this
 * arg or pass an empty string, and the fourth (flags) arg is not 1,
 * then MAPILogon() displays a logon dialog box with an empty name field.
 *
 * The third arg is the password, limited to 256 characters or less. If
 * the messaging system does not require a password, or if it requires
 * that the user enter it, then omit this arg or pass an empty string.
 * When the user must enter a password, you must add in 1 to the fourth
 * arg (flags) to allow a logon dialog box to be displayed.
 *
 * The fourth arg is a flags (numeric) value. It can be any of the following
 * added together:
 *
 * 1 = A logon dialog box should be displayed to prompt for logon information.
 * If the user needs to provide information to enable a successful log on,
 * this value must be added in.
 *
 * 2 = An attempt should be made to create a new session rather than acquire
 * the environment's shared session. If this value is not added in, an
 * existing shared session is used.
 *
 * 8 = Make this a shared session.
 *
 * 16 = Don't use the default profile.
 *
 * 32 = Extended MAPI Logon.
 *
 * 4095 = An attempt should be made to download all of the user's messages
 * before returning. If this is not added in, messages may be downloaded in
 * the background after the call returns.

 * 8192 = Do logon UI in all email accounts.
 *
 * 131072 = Display password dialog only.
 *
 * 1048576 = Minimal wait for logon.
 *
 * The fifth arg can be omitted.
 *
 * The sixth arg is the name of the REXX variable were you want the MAPI
 * "session handle" stored. This will be passed to other MAPI functions.
 */

err = MAPILogon(, , , 1 + 8 /* + 4095 */, , Session)
IF err == 0 THEN DO

	/* Initially, we haven't read an email, so set out email "ID" to an
	 * empty string.
	 */
	msgID = ""

	/* We're going to loop around and check all the unread emails. */
	DO UNTIL msgID = ""

		/* Call MAPIFindNext() to obtain the "ID" of the next unread email
		 * in the default email folder (probably the user's Inbox). The "ID" of
		 * that message is what we will pass to MAPIReadMail() to get the desired
		 * contents of the email.
		 *
		 * The first arg is the Session handle returned by MAPILogon().
		 *
		 * The second arg is a handle to some window you've opened. Omit this
		 * if you haven't opened a window. If you have no window, then any
		 * error message box presented is modal -- that is, the user will have to
		 * dismiss it before he can access another program on his computer.
		 *
		 * The third arg is the name of the message class to search. Some email
		 * clients support several email classes. Omit this to find an interpersonal
		 * message (IPM).
		 *
		 * The fourth arg is the "ID" to start searching from. Omit this, or have
		 * it be an empty string, to start searching from the first message. Here
		 * we'll use an empty string.
		 *
		 * The fifth arg is some flags. Add up any of the following:
		 *
		 * 32 = Read only unread messages. (ie, Not messages that are already read)
		 * 256 = Make MAPIReadMail return the email IDs in the order of time received.
		 *       Some email clients don't support this, in which case MAPIFindNext()
		 *       will return MAPI_E_NO_SUPPORT (ie 268697858).
		 * 16384 = Allow an ID that may have 512 characters in it. Always specify this.
		 *
		 * The sixth arg should always be omitted.
		 *
		 * The seventh arg is where you wish the email ID to be stored. In this
		 * example, we specify a variable named "msgID".
		 */
		err = MAPIFindNext(Session, , , , 32 + 16384, , msgID)

		/* If MAPIFindNext succeeds, we have a the email ID to retrieve the next
		 * unread message. But there might not be any more unread messages in the
		 * folder, so first we test the return value from the MAPIFindNext() to see
		 * if it is MAPI_E_NO_MESSAGES (ie, 16)
		 */
		IF err == 16 THEN DO
			SAY "No more unread emails"
			msgID = ""
		END

		/* Check for some error */
		ELSE IF err \== 0 THEN DO
			SayMAPIErrMessage(err, "Error while searching for messages")
			msgID = ""
		END

		/* Ok, we have another email we can read */
		ELSE DO

			/* No message retrieved yet */
			DROP message
		
			/* Call MAPIReadMail() to fetch the desired contents of the email.
			 * MAPIReadMail will fill in and return a MapiMessage struct
			 * containing the information we request.
			 *
			 * The first arg is the Session handle returned by MAPILogon().
			 *
			 * The second arg is a handle to some window you've opened. Omit this
			 * if you haven't opened a window. See notes above.
			 *
			 * The third arg is the email ID returned by MAPIFindNext().
			 *
			 * The fourth arg is some flags. Add up any of the following:
			 *
			 * 512 = MAPIReadMail should write the message text to a temporary file
			 *       and add it as the first attachment in the MapiMessage struct
			 *       returned to us. The attachments will also be written out to
			 *       temporary files. The returned MapiMessage struct will give the
			 *       names of all those temp files.
			 * 64  = MAPIReadMail should return only the header of the email in the
			 *       MapiMessage struct.
			 * 128 = Do not mark the email as read after returning information.
			 * 2048 = Return only the header, as well as the message body in the
			 *        MapiMessage struct. Do not write any attachments to temp
			 *        files.
			 *
			 * The fifth arg should always be omitted.
			 *
			 * The sixth arg is where we wish MAPIReadMail to store the MapiMessage
			 * struct it allocates and fills in with info about the email. Here we
			 * store it in a variable named "message".
			 */
			err = MAPIReadMail(Session, , msgID, 128 + 2048, , message)

			/* Check for some error */
			IF err \== 0 THEN DO
				SayMAPIErrMessage(err, "Error while retrieving email with ID" msgID)
				msgID = ""
			END

			/* Ok, now we can access the info in the MapiMessage struct. But we
			 * need to use CONVERTDATA to convert it into a format that REXX can
			 * directly access.
			 */

			/* Display the sender's name or address; use the friendly name
			 * if it is present.
			 */
			CONVERTDATA(message+28, err, "struct MapiRecipDesc *")
			IF SYMBOL(err.3) == 'VAR' & err.3 \== "" THEN Sender = err.3 /* We can use the name */
			ELSE Sender = err.4 /* Name is not supplied, so just use the email address */
			SAY "From:" Sender

			/* Display the subject line */
			CONVERTDATA(message+4, Subject, "str *")
			SAY "Subject:" Subject

			/* Display the date received */
			CONVERTDATA(message+16, Date, "str *")
			SAY "Date:" Date

			/* Display all the recipients */
			CONVERTDATA(message+32, count)
			CONVERTDATA(message+36, Recips)         
			DO i = 1 TO count
				CONVERTDATA(Recips, err, "struct MapiRecipDesc *")
				IF SYMBOL(err.3) == 'VAR' & err.3 \== "" THEN Recipient = err.3 /* name */
				ELSE Recipient = err.4 /* Email address */
				SAY "Recipient" i || ":" Recipient
				Recips = Recips + 24
			END
         
			/* Display the message body if there is any */
			CONVERTDATA(message+8, Body, "str *")
			SAY "Body:" Body

			FINALLY
				/* After we're done using the MapiMessage struct that MAPISendMail
				 * allocated to us, we need to free it by calling MAPIFreeBuffer()
				 */
				IF EXISTS(message) THEN MAPIFreeBuffer(message)

		END /* Another email */

	/* Get next email */
	END /* UNTIL msgID = "" */

	FINALLY
		/* Log out
		 *
		 * First arg is the session handle from MAPILogon.
		 *
		 * Second arg is the handle to your window. Omit this if you have no window.
		 */
		MAPILogoff(Session)
END
ELSE SayMAPIErrMessage(err, "Problem logging into MAPI:")

/* Done */
RETURN








SayMAPIErrMessage:
SELECT ARG(1)
	WHEN 1 THEN SAY "User aborted."
	WHEN 3 THEN SAY "There was no default logon, and the user failed to log on successfully when the logon dialog box was displayed."
	WHEN 4 THEN SAY "Can't save attachment because disk is full."
	WHEN 5 THEN SAY "Need more memory to send/receive email."
	WHEN 6 THEN SAY "Access denied."
	WHEN 8 THEN SAY "The user has too many sessions open simultaneously."
	WHEN 9 THEN SAY "Too many attachments."
	WHEN 10 THEN SAY "Too many recipients."
	WHEN 11 THEN SAY "Attachment not found."
	WHEN 12 THEN SAY "Can't open an attachment."
	WHEN 13 THEN SAY "Can't save an attachment to a temporary file."
	WHEN 14 THEN SAY "A recipient did not appear in the address list."
	WHEN 15 THEN SAY "Recipient name is specified incorrectly."
	WHEN 16 THEN SAY "No messages."
	WHEN 17 THEN SAY "Invalid message ID."
	WHEN 18 THEN SAY "The text in the message was too large to send."
	WHEN 19 THEN SAY "Invalid session."
	WHEN 20 THEN SAY "Type not supported."
	WHEN 21 THEN SAY "A recipient was specified more than once."
	WHEN 22 THEN SAY "Message in use."
	WHEN 23 THEN SAY "Network failure."
	WHEN 25 THEN SAY "Invalid recipients."
	OTHERWISE SAY ARG(2) err
END
RETURN